# -*- coding: utf-8 -*-
"""Python lager brewed by a loguru"""
from lager import logging
from lager.__about__ import __version__
from lager.const import LAGER_PORT, LOGURU_DEFAULT_FMT, TORNADO_FMT
from lager.core import (
    LAGER,
    LN,
    LOG,
    _change_activation,
    _core,
    _find_iter,
    _log,
    _options,
    add,
    bind,
    catch,
    complete,
    configure,
    contextualize,
    critical,
    debug,
    disable,
    enable,
    error,
    exception,
    flog,
    handlers,
    info,
    lager,
    level,
    ln,
    log,
    logger,
    loglevel,
    opt,
    parse,
    patch,
    remove,
    reset,
    start,
    stop,
    success,
    trace,
    warning,
)

__all__ = (
    "__version__",
    "LAGER_PORT",
    "LOGURU_DEFAULT_FMT",
    "TORNADO_FMT",
    "loglevel",
    "handlers",
    "flog",
    "logger",
    "logging",
    "lager",
    "LAGER",
    "LOG",
    "log",
    "ln",
    "LN",
    "reset",
    #############
    ## HOISTED ##
    #############
    "_change_activation",
    "_core",
    "_find_iter",
    "_log",
    "_options",
    "add",
    "bind",
    "catch",
    "complete",
    "configure",
    "contextualize",
    "critical",
    "debug",
    "disable",
    "enable",
    "error",
    "exception",
    "info",
    "level",
    "opt",
    "parse",
    "patch",
    "remove",
    "start",
    "stop",
    "success",
    "trace",
    "warning",
)



# -*- coding: utf-8 -*-
"""FastAPI logging"""
from typing import List, Optional, Set, Tuple, Union

from lager.logging import intercept

__all__ = ("FASTAPI_LOGGERS", "fastapi_intercept")

FASTAPI_LOGGERS = [
    "gunicorn",
    "gunicorn.errors" "uvicorn",
    "uvicorn.error",
    "fastapi",
    "sqlalchemy",
]


def fastapi_intercept(
    loggers: Optional[Union[List[str], Set[str], Tuple[str, ...]]] = None
) -> None:
    _loggers2intercept = (
        FASTAPI_LOGGERS if not loggers else sorted(set(*(*FASTAPI_LOGGERS, loggers)))
    )
    intercept(_loggers2intercept)



# -*- coding: utf-8 -*-
"""Package metadata/info"""

__all__ = ("__title__", "__description__", "__pkgroot__", "__version__")
__title__ = "lager"
__description__ = "EZ-PZ logging based on loguru"
__pkgroot__ = __file__.replace("__about__.py", "").rstrip("/\\")
__version__ = "0.17.2"



# -*- coding: utf-8 -*-

from typing import Any

try:
    import httpx

    _SINKS = []

    class HttpxSink(object):
        def __init__(self, url: str, *args: Any, **kwargs: Any) -> None:
            self.url = url
            self.client = httpx.AsyncClient(*args, **kwargs)
            _SINKS.append(self)

        async def __call__(self, msg: Any) -> None:
            httpx.post(self.url, data={"msg": msg})

        async def handle(self, message: Any) -> None:
            await self.client.post(url=self.url, data={"msg": message})

        async def await_delete_channels(self) -> None:
            await self.client.aclose()

except ImportError:
    pass



# -*- coding: utf-8 -*-
"""Lager/Loguru + std logging"""
import logging

from logging import (
    BASIC_FORMAT as BASIC_FORMAT,
    CRITICAL as CRITICAL,
    DEBUG as DEBUG,
    ERROR as ERROR,
    FATAL as FATAL,
    INFO as INFO,
    NOTSET as NOTSET,
    WARN as WARN,
    WARNING as WARNING,
    BufferingFormatter as BufferingFormatter,
    FileHandler as FileHandler,
    Filter as Filter,
    Formatter as Formatter,
    Handler as Handler,
    Logger as Logger,
    LoggerAdapter as LoggerAdapter,
    LogRecord as LogRecord,
    NullHandler as NullHandler,
    StreamHandler as StreamHandler,
    addLevelName as addLevelName,
    addLevelName as add_level_name,
    basicConfig as basicConfig,
    basicConfig as basic_config,
    captureWarnings as captureWarnings,
    captureWarnings as capture_warnings,
    critical as critical,
    debug as debug,
    disable as disable,
    error as error,
    exception as exception,
    fatal as fatal,
    getLevelName as getLevelName,
    getLevelName as get_level_name,
    getLogger as getLogger,
    getLogger as get_logger,
    getLoggerClass as getLoggerClass,
    getLoggerClass as get_logger_class,
    getLogRecordFactory as getLogRecordFactory,
    getLogRecordFactory as get_log_record_factory,
    info as info,
    lastResort as lastResort,
    lastResort as last_resort,
    log as log,
    makeLogRecord as makeLogRecord,
    makeLogRecord as make_log_record,
    raiseExceptions as raiseExceptions,
    raiseExceptions as raise_exceptions,
    setLoggerClass as setLoggerClass,
    setLoggerClass as set_logger_class,
    setLogRecordFactory as setLogRecordFactory,
    setLogRecordFactory as set_log_record_factory,
    shutdown as shutdown,
    warn as warn,
    warning as warning,
)
from typing import Any, Dict, List

from lager.core import LOG, loglevel

__all__ = (
    "BASIC_FORMAT",
    "BufferingFormatter",
    "CRITICAL",
    "DEBUG",
    "ERROR",
    "FATAL",
    "FileHandler",
    "Filter",
    "Formatter",
    "Handler",
    "INFO",
    "LogRecord",
    "Logger",
    "LoggerAdapter",
    "NOTSET",
    "NullHandler",
    "StreamHandler",
    "WARN",
    "WARNING",
    "addLevelName",
    "add_level_name",
    "basicConfig",
    "basic_config",
    "captureWarnings",
    "capture_warnings",
    "critical",
    "debug",
    "disable",
    "error",
    "exception",
    "fatal",
    "getLevelName",
    "getLogRecordFactory",
    "getLogger",
    "getLoggerClass",
    "get_level_name",
    "get_log_record_factory",
    "get_logger",
    "get_logger_class",
    "info",
    "lastResort",
    "last_resort",
    "log",
    "makeLogRecord",
    "make_log_record",
    "raiseExceptions",
    "raise_exceptions",
    "setLogRecordFactory",
    "setLoggerClass",
    "set_log_record_factory",
    "set_logger_class",
    "shutdown",
    "warn",
    "warning",
    # lager.logging members
    "__aliases__",
    "StdLoggingHandler",
    "loggers_dict",
    "intercept",
    "intercept_all",
    "patch_logging",
)

# logging snake_case aliases bc I cannot stand camelCase
__aliases__ = {
    "add_level_name": "addLevelName",
    "basic_config": "basicConfig",
    "capture_warnings": "captureWarnings",
    "get_level_name": "getLevelName",
    "get_log_record_factory": "getLogRecordFactory",
    "get_logger": "getLogger",
    "get_logger_class": "getLoggerClass",
    "last_resort": "lastResort",
    "make_log_record": "makeLogRecord",
    "raise_exceptions": "raiseExceptions",
    "set_log_record_factory": "setLogRecordFactory",
    "set_logger_class": "setLoggerClass",
}


def patch_logging() -> None:
    for k, v in __aliases__.items():
        setattr(logging, k, getattr(logging, v))


class StdLoggingHandler(logging.Handler):
    """Logging intercept handler"""

    def emit(self, record: logging.LogRecord) -> None:
        # Get corresponding Loguru level if it exists
        try:
            level = LOG.level(record.levelname).name
        except AttributeError:
            level = loglevel(record.levelno)

        # Find caller from where originated the logging call
        frame = logging.currentframe()
        depth = 2
        while (
            frame.f_code.co_filename
            == logging.__file__  # pyright: ignore[reportOptionalMemberAccess]
        ):
            frame = frame.f_back  # type: ignore[assignment]
            depth += 1

        LOG.opt(depth=depth, exception=record.exc_info).log(level, record.getMessage())


def _logger_dict() -> Any:
    return logging.root.manager.loggerDict


def loggers_dict() -> Dict[str, logging.Logger]:
    return {name: logging.getLogger(name) for name in _logger_dict()}


def intercept(loggers: List[str]) -> None:
    for logger in loggers:
        std_logger = logging.getLogger(logger)
        std_logger.handlers = [StdLoggingHandler()]


def intercept_all() -> None:
    intercept(list(loggers_dict().keys()))



# -*- coding: utf-8 -*-
"""Python lager brewed by a loguru"""
import asyncio
import sys as _sys

from functools import wraps
from time import time
from typing import Any, Callable, Dict, Optional, TypeVar, Union

from loguru import logger as logger
from loguru._handler import Handler

from lager.const import LOG_LEVELS

T = TypeVar("T")

try:
    import orjson

    def _stringify_new_line(serializable: Any) -> str:
        return orjson.dumps(serializable, option=orjson.OPT_APPEND_NEWLINE).decode(
            "utf-8"
        )

    def _stringify_no_new_line(serializable: Any) -> str:
        return f"{orjson.dumps(serializable).decode('utf-8')}\n"

    _stringify = (
        _stringify_new_line
        if hasattr(orjson, "OPT_APPEND_NEWLINE")
        else _stringify_no_new_line
    )

    def _serialize_record(text: str, record: Dict[str, Any]) -> str:
        exception = record["exception"]

        if exception is not None:
            exception = {
                "type": None if exception.type is None else exception.type.__name__,
                "value": exception.value,
                "traceback": bool(record["exception"].traceback),
            }

        serializable = {
            "text": text,
            "record": {
                "elapsed": {
                    "repr": record["elapsed"],
                    "seconds": record["elapsed"].total_seconds(),
                },
                "exception": exception,
                "extra": record["extra"],
                "file": {
                    "name": record["file"].name,
                    "path": record["file"].path,
                },
                "function": record["function"],
                "level": {
                    "icon": record["level"].icon,
                    "name": record["level"].name,
                    "no": record["level"].no,
                },
                "line": record["line"],
                "message": record["message"],
                "module": record["module"],
                "name": record["name"],
                "process": {
                    "id": record["process"].id,
                    "name": record["process"].name,
                },
                "thread": {
                    "id": record["thread"].id,
                    "name": record["thread"].name,
                },
                "time": {
                    "repr": record["time"],
                    "timestamp": record["time"].timestamp(),
                },
            },
        }
        return _stringify(serializable)

    Handler._serialize_record = staticmethod(_serialize_record)
except ModuleNotFoundError:
    pass

# lager/logger aliases
log = LOG = logger
lager = LAGER = logger
ln = LN = logger  # ln => natural log


def loglevel(level: Union[str, int]) -> str:
    """Convert log-level abrev to a valid loguru log level"""
    return str(LOG_LEVELS[str(level).strip("'").strip('"').lower()])


def flog(
    funk: Optional[Callable[..., T]] = None,
    level: Union[str, int] = "debug",
    enter: bool = True,
    exit: bool = True,
) -> T:
    """Log function (sync/async) enter and exit using this decorator

    Args:
        funk (Callable): Function to decorate
        level (Union[int, str]): Log level
        enter (bool): Log function entry if True
        exit (bool): Log function exit if False

    Returns:
        A wrapped function that now has logging!

    Usage:
        # SYNC
        @flog
        def add(a, b):
            return a + b
        add(1, 4)

        # ASYNC
        @flog
        async def add_async(a, b):
            return a + b
        import asyncio
        asyncio.run(add_async(1, 4))

    """

    def _flog(funk: Callable[..., T]) -> Callable[..., T]:
        name = funk.__name__

        @wraps(funk)
        def _flog_decorator(*args: Any, **kwargs: Any) -> T:
            logger_ = logger.opt(depth=1)
            if enter:
                logger_.log(
                    loglevel(level),
                    "FLOG-ENTER > '{}' (args={}, kwargs={})",
                    name,
                    args,
                    kwargs,
                )
            ti = time()
            result = funk(*args, **kwargs)
            tf = time()
            if exit:
                logger_.log(
                    loglevel(level),
                    "FLOG-EXIT < '{}' (return={}, dt_sec={})",
                    name,
                    result,
                    tf - ti,
                )
            return result

        @wraps(funk)
        async def _flog_decorator_async(*args: Any, **kwargs: Any) -> T:
            logger_ = logger.opt(depth=7)
            if enter:
                logger_.log(
                    loglevel(level),
                    "FLOG-ENTER > '{}' (args={}, kwargs={})",
                    name,
                    args,
                    kwargs,
                )
            ti = time()
            result: T = await funk(*args, **kwargs)  # type: ignore[misc]
            tf = time()
            if exit:
                logger_.log(
                    loglevel(level),
                    "FLOG-EXIT < '{}' (return={}, dt_sec={})",
                    name,
                    result,
                    tf - ti,
                )
            return result

        if asyncio.iscoroutinefunction(funk) or asyncio.iscoroutine(funk):
            return _flog_decorator_async  # type: ignore[return-value]
        return _flog_decorator

    return _flog(funk) if funk else _flog  # type: ignore[return-value]


def handlers() -> Dict[int, Handler]:
    """Return all handlers"""
    return logger._core.handlers  # type: ignore[no-any-return]


def reset(level: Optional[Union[str, int]] = None) -> None:
    logger.remove()
    logger.add(_sys.stderr, level=loglevel(level or "debug"))


__hoisted__ = (
    "_change_activation",
    "_core",
    "_dynamic_level",
    "_find_iter",
    "_log",
    "_options",
    "add",
    "bind",
    "catch",
    "complete",
    "configure",
    "contextualize",
    "critical",
    "debug",
    "disable",
    "enable",
    "error",
    "exception",
    "info",
    "level",
    "opt",
    "parse",
    "patch",
    "remove",
    "start",
    "stop",
    "success",
    "trace",
    "warning",
)
_change_activation = LAGER._change_activation
_core = LAGER._core
_find_iter = LAGER._find_iter
_log = LAGER._log
_options = LAGER._options
add = LAGER.add
bind = LAGER.bind
catch = LAGER.catch
complete = LAGER.complete
configure = LAGER.configure
contextualize = LAGER.contextualize
critical = LAGER.critical
debug = LAGER.debug
disable = LAGER.disable
enable = LAGER.enable
error = LAGER.error
exception = LAGER.exception
info = LAGER.info
level = LAGER.level
opt = LAGER.opt
parse = LAGER.parse
patch = LAGER.patch
remove = LAGER.remove
start = LAGER.start
stop = LAGER.stop
success = LAGER.success
trace = LAGER.trace
warning = LAGER.warning

__all__ = (
    "LAGER",
    "LOG",
    "_change_activation",
    "_core",
    "_find_iter",
    "_log",
    "_options",
    "add",
    "bind",
    "catch",
    "complete",
    "configure",
    "contextualize",
    "critical",
    "debug",
    "disable",
    "enable",
    "error",
    "exception",
    "flog",
    "handlers",
    "info",
    "lager",
    "level",
    "ln",
    "log",
    "logger",
    "loglevel",
    "opt",
    "parse",
    "patch",
    "remove",
    "start",
    "stop",
    "success",
    "trace",
    "warning",
)



# -*- coding: utf-8 -*-
"""Package metadata/info"""
import warnings

from lager.__about__ import __description__, __pkgroot__, __title__, __version__

warnings.warn(
    "lager._meta is deprecated, use lager.__about__ instead",
    DeprecationWarning,
    stacklevel=2,
)

__all__ = ("__title__", "__description__", "__pkgroot__", "__version__")
